class AlgorithmsGCD() :

    """Cette classe contient les algorithmes de calcul du 
    PGCD, tout a ete fait pour que garder la genericite.
    Les 2 algorithmes ci desous sont utilises pour calculer
    le pgcd et les coefficients de Bezout dans l'anneau
    des entiers ou des polynomes"""

    def __init__(self, logname) :
        """Construit un nouvel objet de calcul de PGCD"""
        self.logger = logname

    def EuclideanAlgorithm(self, A, B) :
        """Calcule le pgcd par l'algorithme d'Euclide"""
        # Initialise les variables
        quotient, reste = A, B
        # Si un element est deja le neutre, il n'y a rien
        # a faire, a part eviter les divisions par 0 !
        if quotient.isNull() :
            return reste
        if reste.isNull() :
            return quotient

        # On repete la division euclidienne jusqu'a obtenir
        # l'element neutre de l'addition
        # Pour sortie latex :
        while not(reste.isNull()) :
            q, r = quotient.EuclideanDivision(reste)

            # Prepare l'iteration suivante en placant
            # "quotient" dans "reste" et le "r" dans "reste"
            quotient = reste
            reste = r

        # Simplifie le pgcd
        quotient = quotient.simplify_pgcd()
        return quotient

    def ExtendedEuclideanAlgorithm(self, A, B) :
        """Calcule le pgcd et les coefficients de Bezout
        par l'algorithme d'Euclide etendu"""
        quotient, reste = A, B
        # Si un element est deja le neutre, il n'y a rien
        # a faire, a part eviter les divisions par 0 !
        if quotient.isNull() and reste.isNull() :
            # Les 2 elements sont nulls =>
            # pgcd et coeffs sont nulls !
            return reste.getAdditionNeutral(), ##
                   quotient.getAdditionNeutral(), ##
                   quotient.getAdditionNeutral()

        # Traitement des cas particulier : A ou B est nul
        if quotient.isNull() :
            return reste, reste.getAdditionNeutral(),##
                   reste.getMultiplicationNeutral()

        if reste.isNull() :
            return quotient, quotient.getMultiplicationNeutral(),##
                   quotient.getAdditionNeutral()
    
        # Initialise les coef de Bezout
        oldCoefA = quotient.getMultiplicationNeutral()
        oldCoefB = quotient.getAdditionNeutral()
        coefA = quotient.getAdditionNeutral()
        coefB = quotient.getMultiplicationNeutral()
        
         # On repete la division euclidienne jusqu'a
        # obtenir l'element neutre de l'addition
        while not(reste.isNull()) :
            q, r = quotient.EuclideanDivision(reste)
            # Calcule les nouveaux coefs
            tmpCoefA = oldCoefA - q * coefA
            tmpCoefB = oldCoefB - q * coefB
            # "Rotation" des coefs
            oldCoefA = coefA
            oldCoefB = coefB

            coefA = tmpCoefA
            coefB = tmpCoefB

            # Prepare l'iteration suivante en placant
            # "quotient" dans "reste" et le "r" dans "reste"
            quotient = reste
            reste = r

        # Simplifie le pgcd et met a jour les coefs
        # de Bezout associes
        quotient, oldCoefA, oldCoefB = ##
             quotient.simplify_pgcd_and_coeffs(oldCoefA, oldCoefB)
        return quotient, oldCoefA, oldCoefB

