from sympy import *

class PolynomialRing() :

    """"Cette classe contient tout ce dont nous avons besoin 
    pour calculer un pgcd dans l'anneau des polynomes"""
    
    ## Le "vrai but de cette classe est de faire le lien entre
    ## python et XCAS au travers du module sympy.
    ## poly est la variable sympy contenant le polynome
    poly = Symbol('poly')

    def __init__(self, latex_string) :
        """Construit un nouveau polynome, en utilisant pour
        l'initialiser soit une chaine de caracteres au format
	sympy."""
        ## On declare une variable 'x' car nos polynomes
        ## utiliseront cette inconnu
        X = Symbol('X')
        ## Transforme la chaine de caracteres au format
        ## Latex en une chaine au format sympy
        sympy_string = latex_string.replace("^", "**")
        sympy_string = sympy_string.replace("i", "I")
        sympy_string = sympy_string.replace("x", "X")
        ## Cree le polynome demande
        self.poly = eval("Poly(" + sympy_string + " , X)")
        
    def EuclideanDivision(self, B) :
        """Calcule la division euclidienne de self par B
        et retourne le quotient et le reste."""
        ## La division est faite par XCAS, on masque cela a
        ## l'algorithme de calcul du pgcd pour faire ressortir
        ## sa genericite
        X = Symbol('X')
        q, r = div(self.poly, B.poly, X)
        quo = PolynomialRing("0")
        rem = PolynomialRing("0")
        quo.poly = q
        rem.poly = r
        return quo, rem

    def getAdditionNeutral(self) :
        """Retourne l'element neutre de l'addition."""
        return PolynomialRing("0")

    def getMultiplicationNeutral(self) :
        """Retourne l'element neutre de la multiplication."""
        return PolynomialRing("1")

    def isNull(self) :
        """Retourne True si self est egal a l'element
        neutre de l'addition"""
        if self.poly.as_dict() == {} :
            return True
        else :
            return False

    def simplify(self) :
        """Simplifie le polynome (i.e. apres certains calculs les
        coefficients complexes du polynomes ne sont pas implicitement
        simplifies. Cette fonction permet de le faire)."""
        # Je ne sais pas pourquoi mais il y a un bug dans la fonction
        # simplify du module sympy : si le polynome est constant, une
        # erreur survient. Cette fonction n'a pas d'autre but que de
        # contourner ce probleme
        X = Symbol('X')
        if self.poly.degree() == 0 :
            # Si le polynome est constant, on recupere ce terme constant,
            # on le simplifie et le retourne sous forme de polynome
            tmpCoef = self.poly.LC()
            tmpCoef = tmpCoef.simplify()
            self.poly = eval("Poly(" + str(tmpCoef) + " , X)")
            return self
        self.poly = self.poly.simplify()
        return self
        
    def simplify_pgcd(self) :
        """Met le pgcd trouve sous forme unitaire"""
        self.poly = self.poly.monic()
        return self.simplify()

    def simplify_pgcd_and_coeffs(self, coefA, coefB) :
        """Met le pgcd trouve sous forme unitaire et divise les
        coefs de Bezout par le meme facteur"""
        ## On recherche le coef de plus haut niveau
        X = Symbol('X')
        high_coef = self.poly.LC()
        cst = eval("Poly(" + str(high_coef) + " , X)")
        ## On met le pgcd sous forme unitaire
        self.poly = self.poly.monic()
        self = self.simplify()
        ##  On divise les coeffs associes par le coef
        ## de plus haut degre du pgcd
        tmpPoly = coefA.poly.div(cst)
        ca = PolynomialRing("0")
        ca.poly = tmpPoly[0]
        ca = ca.simplify()
        tmpPoly = coefB.poly.div(cst)
        cb = PolynomialRing("0")
        cb.poly = tmpPoly[0]
        cb = cb.simplify()
        return self, ca, cb

    def __add__(self, B) :
        """Additionne deux polynomes"""
        ## L'addition est faite par XCAS, mais l'algorithme
        ## du pgcd ne le sait pas.
        result = PolynomialRing("0")
        result.poly = self.poly + B.poly
        return result

    def __sub__(self, B) :
        """Soustrait B a self"""
        ## La soustraction est faite par XCAS, mais l'algorithme
        ## du pgcd ne le sait pas.
        result = PolynomialRing("0")
        result.poly = self.poly - B.poly
        return result

    def __mul__(self, B) :
        """Multiplie deux polynomes"""
        ## La multiplication est faite par XCAS, mais l'algorithme
        ## du pgcd ne le sait pas.
        result = PolynomialRing("0")
        result.poly = self.poly * B.poly
        return result


